@@ -20,7 +20,7 @@
  * SECTION: pt-player
  * @short_description: The GStreamer backend for Parlatype.
  * @stability: Stable
- * @include: parlatype/pt-player.h
+ * @include: parlatype/parlatype.h
  *
  * PtPlayer is the GStreamer backend for Parlatype. Construct it with #pt_player_new().
  * Then you have to open a file with pt_player_open_uri().
@@ -38,11 +38,16 @@
 #include "gst/gstptaudiobin.h"
 #include "pt-config.h"
 #include "pt-i18n.h"
+#include "pt-media-info-private.h"
+#include "pt-media-info.h"
 #include "pt-position-manager.h"
 #include "pt-waveviewer.h"
 #ifdef HAVE_POCKETSPHINX
 #include "gst/gstparlasphinx.h"
 #endif
+#ifdef HAVE_POCKETSPHINX_LEGACY
+#include "gst/gstparlasphinx-legacy.h"
+#endif
 
 #include <gio/gio.h>
 #include <glib/gi18n-lib.h>
@@ -57,6 +62,11 @@ struct _PtPlayerPrivate
   GstElement *audio_bin;
   guint       bus_watch_id;
 
+  PtMediaInfo         *info;
+  GstStreamCollection *collection;
+  gchar               *stream_id;
+  gulong               stream_notify_id;
+
   PtPositionManager *pos_mgr;
   GHashTable        *plugins;
 
@@ -80,10 +90,13 @@ struct _PtPlayerPrivate
   gboolean repeat_all;
   gboolean repeat_selection;
 
-  gint64 segstart;
-  gint64 segend;
+  gint64       segstart;
+  GstClockTime segend;
 
   GCancellable *c;
+  guint         vol_changed_id;
+  guint         mute_changed_id;
+  guint         uri_changed_id;
 
   PtPrecisionType timestamp_precision;
   gboolean        timestamp_fixed;
@@ -120,7 +133,6 @@ static GParamSpec *obj_properties[N_PROPERTIES];
 #define TEN_MINUTES 600000
 
 static gboolean bus_call (GstBus *bus, GstMessage *msg, gpointer data);
-static void     remove_message_bus (PtPlayer *self);
 
 G_DEFINE_TYPE_WITH_PRIVATE (PtPlayer, pt_player, G_TYPE_OBJECT)
 
@@ -131,7 +143,7 @@ pt_player_clear (PtPlayer *self)
 {
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
 
-  remove_message_bus (self);
+  g_clear_handle_id (&priv->bus_watch_id, g_source_remove);
   priv->target_state = GST_STATE_NULL;
   priv->current_state = GST_STATE_NULL;
   gst_element_set_state (priv->play, GST_STATE_NULL);
@@ -317,7 +329,7 @@ metadata_goto_position (PtPlayer *self)
   g_object_unref (file);
 }
 
-const static gchar *
+static const gchar *
 pt_player_get_state_name (PtStateType state)
 {
   switch (state)
@@ -330,9 +342,10 @@ pt_player_get_state_name (PtStateType state)
       return "playing";
     case PT_STATE_INVALID:
       return NULL;
+    default:
+      g_assert_not_reached ();
+      return NULL;
     }
-
-  return NULL;
 }
 
 static void
@@ -356,27 +369,46 @@ change_app_state (PtPlayer   *self,
 }
 
 static void
-remove_message_bus (PtPlayer *self)
+stream_notify_cb (GstStreamCollection *collection,
+                  GstStream           *stream,
+                  GParamSpec          *pspec,
+                  gpointer             user_data)
 {
+  PtPlayer        *self = (PtPlayer *) user_data;
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+  const gchar     *stream_id;
+  gboolean         our_stream;
+  gboolean         tags_changed;
 
-  if (priv->bus_watch_id > 0)
-    {
-      g_source_remove (priv->bus_watch_id);
-      priv->bus_watch_id = 0;
-    }
+  tags_changed = (G_PARAM_SPEC_VALUE_TYPE (pspec) == GST_TYPE_TAG_LIST);
+  stream_id = gst_stream_get_stream_id (stream);
+
+  g_mutex_lock (&priv->lock);
+  our_stream = (g_strcmp0 (priv->stream_id, stream_id) == 0);
+
+  if (our_stream && tags_changed)
+    _pt_media_info_update_tags (priv->info, gst_stream_get_tags (stream));
+  g_mutex_unlock (&priv->lock);
 }
 
 static void
-add_message_bus (PtPlayer *self)
+update_stream_collection (PtPlayer            *self,
+                          GstStreamCollection *collection)
 {
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
-  GstBus          *bus;
 
-  remove_message_bus (self);
-  bus = gst_pipeline_get_bus (GST_PIPELINE (priv->play));
-  priv->bus_watch_id = gst_bus_add_watch (bus, bus_call, self);
-  gst_object_unref (bus);
+  if (priv->collection && priv->collection == collection)
+    return;
+
+  g_clear_signal_handler (&priv->stream_notify_id, priv->collection);
+
+  gst_object_replace ((GstObject **) &priv->collection,
+                      (GstObject *) collection);
+
+  priv->stream_notify_id = g_signal_connect (priv->collection,
+                                             "stream-notify",
+                                             G_CALLBACK (stream_notify_cb),
+                                             self);
 }
 
 static gboolean
@@ -395,7 +427,7 @@ bus_call (GstBus     *bus,
       /* Sometimes the current position is not exactly at the end which looks like
        * a premature EOS or makes the cursor disappear.*/
       gst_element_query_position (priv->play, GST_FORMAT_TIME, &pos);
-      stop = GST_CLOCK_TIME_IS_VALID (priv->segend) ? priv->segend : priv->dur;
+      stop = GST_CLOCK_TIME_IS_VALID (priv->segend) ? (gint64) priv->segend : priv->dur;
       if (pos != stop)
         {
           g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
@@ -412,13 +444,11 @@ bus_call (GstBus     *bus,
         gst_element_query_duration (priv->play, GST_FORMAT_TIME, &dur);
         g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "MESSAGE",
                           "New duration: %" GST_TIME_FORMAT, GST_TIME_ARGS (dur));
-        if (priv->dur != priv->segend)
+        priv->dur = dur;
+        if (GST_CLOCK_TIME_IS_VALID (priv->segend) &&
+            priv->segend > (guint64) dur)
           {
-            priv->dur = dur;
-          }
-        else
-          {
-            priv->dur = priv->segend = dur;
+            priv->segend = dur;
             gst_element_query_position (priv->play, GST_FORMAT_TIME, &pos);
             pt_player_seek (self, pos);
           }
@@ -542,6 +572,47 @@ bus_call (GstBus     *bus,
         }
       break;
 
+    case GST_MESSAGE_STREAMS_SELECTED:;
+      GstStreamCollection *collection = NULL;
+      gboolean             found_stream = FALSE;
+      guint                i, len;
+
+      gst_message_parse_streams_selected (msg, &collection);
+      if (collection == NULL)
+        break;
+
+      g_mutex_lock (&priv->lock);
+      update_stream_collection (self, collection);
+      g_mutex_unlock (&priv->lock);
+
+      gst_object_unref (collection);
+
+      len = gst_message_streams_selected_get_size (msg);
+      for (i = 0; i < len; i++)
+        {
+          GstStream    *stream;
+          GstStreamType stream_type;
+          stream = gst_message_streams_selected_get_stream (msg, i);
+          stream_type = gst_stream_get_stream_type (stream);
+          if ((stream_type & GST_STREAM_TYPE_AUDIO) == 0)
+            continue;
+
+          if (G_UNLIKELY (found_stream))
+            {
+              GST_FIXME_OBJECT (self, "Multiple audio streams selected, first one chosen");
+              continue;
+            }
+
+          found_stream = TRUE;
+          g_mutex_lock (&priv->lock);
+          g_free (priv->stream_id);
+          priv->stream_id = g_strdup (gst_stream_get_stream_id (stream));
+          _pt_media_info_update_caps (priv->info, gst_stream_get_caps (stream));
+          _pt_media_info_update_tags (priv->info, gst_stream_get_tags (stream));
+          g_mutex_unlock (&priv->lock);
+        }
+      break;
+
     default:
       break;
     }
@@ -582,6 +653,7 @@ pt_player_open_uri (PtPlayer *self,
   g_return_val_if_fail (uri != NULL, FALSE);
 
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+  GstBus          *bus;
 
   /* If we had an open file before, remember its position */
   metadata_save_position (self);
@@ -593,7 +665,9 @@ pt_player_open_uri (PtPlayer *self,
   g_object_set (G_OBJECT (priv->play), "uri", uri, NULL);
 
   /* setup message handler */
-  add_message_bus (self);
+  bus = gst_pipeline_get_bus (GST_PIPELINE (priv->play));
+  priv->bus_watch_id = gst_bus_add_watch (bus, bus_call, self);
+  gst_object_unref (bus);
 
   pt_player_pause (self);
 
@@ -770,18 +844,14 @@ pt_player_play_pause (PtPlayer *self)
 
   switch (priv->current_state)
     {
-    case GST_STATE_NULL:
-      return;
     case GST_STATE_PAUSED:
       pt_player_play (self);
       break;
     case GST_STATE_PLAYING:
       pt_player_pause_and_rewind (self);
       break;
-    case GST_STATE_VOID_PENDING:
-      return;
-    case GST_STATE_READY:
-      return;
+    default:
+      break;
     }
 }
 
@@ -805,8 +875,15 @@ pt_player_set_selection (PtPlayer *self,
 {
   g_return_if_fail (PT_IS_PLAYER (self));
   g_return_if_fail (start < end);
+  g_return_if_fail (start >= 0);
+  g_return_if_fail (end >= 0);
 
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+
+  if (priv->segstart == GST_MSECOND * start &&
+      priv->segend == GST_MSECOND * (guint64) end)
+    return;
+
   priv->segstart = GST_MSECOND * start;
   priv->segend = GST_MSECOND * end;
 
@@ -815,7 +892,7 @@ pt_player_set_selection (PtPlayer *self,
   if (!gst_element_query_position (priv->play, GST_FORMAT_TIME, &pos))
     return;
 
-  if (pos < priv->segstart || pos > priv->segend)
+  if (pos < priv->segstart || (guint64) pos > priv->segend)
     pos = priv->segstart;
 
   pt_player_seek (self, pos);
@@ -837,6 +914,10 @@ pt_player_clear_selection (PtPlayer *self)
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   gint64           pos;
 
+  if (priv->segstart == 0 &&
+      priv->segend == GST_CLOCK_TIME_NONE)
+    return;
+
   priv->segstart = 0;
   priv->segend = GST_CLOCK_TIME_NONE;
 
@@ -894,7 +975,7 @@ pt_player_jump_relative (PtPlayer *self,
   if (!gst_element_query_position (priv->play, GST_FORMAT_TIME, &pos))
     return;
 
-  new = pos + GST_MSECOND *(gint64) milliseconds;
+  new = pos + GST_MSECOND *milliseconds;
   g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
                     "MESSAGE", "Jump relative: dur = %" GST_TIME_FORMAT, GST_TIME_ARGS (priv->dur));
   g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
@@ -903,7 +984,7 @@ pt_player_jump_relative (PtPlayer *self,
   if (new > priv->dur)
     new = priv->dur;
 
-  if (GST_CLOCK_TIME_IS_VALID (priv->segend) && new > priv->segend)
+  if (GST_CLOCK_TIME_IS_VALID (priv->segend) && (guint64) new > priv->segend)
     new = priv->segend;
 
   if (new < priv->segstart)
@@ -1007,7 +1088,7 @@ pt_player_jump_to_position (PtPlayer *self,
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   gint64           pos;
 
-  pos = GST_MSECOND * (gint64) milliseconds;
+  pos = GST_MSECOND * milliseconds;
 
   if (pos < 0)
     {
@@ -1023,7 +1104,7 @@ pt_player_jump_to_position (PtPlayer *self,
                         GST_TIME_ARGS (pos), GST_TIME_ARGS (priv->segstart));
       return;
     }
-  if (GST_CLOCK_TIME_IS_VALID (priv->segend) && pos > priv->segend)
+  if (GST_CLOCK_TIME_IS_VALID (priv->segend) && (guint64) pos > priv->segend)
     {
       g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
                         "MESSAGE", "Setting position failed: target %" GST_TIME_FORMAT " after segend %" GST_TIME_FORMAT,
@@ -1279,37 +1360,18 @@ static void
 wv_selection_changed_cb (GtkWidget *widget,
                          PtPlayer  *self)
 {
-  /* Selection changed in Waveviewer widget:
-     - if we are not playing a selection: just set start/stop without seeking
-     - if we are playing a selection and we are still in the selection:
-       update selection
-     - if we are playing a selection and the new one is somewhere else:
-       stop playing the selection */
-
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
-  gint64           start, end, pos;
+  gint64           start, end;
+
   g_object_get (priv->wv,
                 "selection-start", &start,
                 "selection-end", &end,
                 NULL);
 
-  if (priv->current_state == GST_STATE_PAUSED || !GST_CLOCK_TIME_IS_VALID (priv->segend))
-    {
-      priv->segstart = GST_MSECOND * start;
-      priv->segend = (end == 0) ? GST_CLOCK_TIME_NONE : GST_MSECOND * end;
-      return;
-    }
-
-  if (!gst_element_query_position (priv->play, GST_FORMAT_TIME, &pos))
-    return;
-  if (GST_MSECOND * start <= pos && pos <= GST_MSECOND * end)
-    {
-      pt_player_set_selection (self, start, end);
-    }
+  if (start == end)
+    pt_player_clear_selection (self);
   else
-    {
-      pt_player_clear_selection (self);
-    }
+    pt_player_set_selection (self, start, end);
 }
 
 static void
@@ -1821,6 +1883,9 @@ pt_player_get_timestamp_position (PtPlayer *self,
                                   gchar    *timestamp,
                                   gboolean  check_duration)
 {
+  g_return_val_if_fail (PT_IS_PLAYER (self), -1);
+  g_return_val_if_fail (timestamp != NULL, -1);
+
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   gint             h, m, s, ms, result;
   gchar           *cmp; /* timestamp without delimiters */
@@ -1988,12 +2053,10 @@ pt_player_goto_timestamp (PtPlayer *self,
 
 /* --------------------- Init and GObject management ------------------------ */
 
-static gboolean
-notify_volume_idle_cb (PtPlayer *self)
+static void
+notify_volume_idle_cb (gpointer user_data)
 {
-  if (!PT_IS_PLAYER (self))
-    return G_SOURCE_REMOVE;
-
+  PtPlayer        *self = PT_PLAYER (user_data);
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   gdouble          vol;
 
@@ -2006,7 +2069,7 @@ notify_volume_idle_cb (PtPlayer *self)
                                 obj_properties[PROP_VOLUME]);
     }
 
-  return FALSE;
+  priv->vol_changed_id = 0;
 }
 
 static void
@@ -2017,17 +2080,15 @@ vol_changed (GObject    *object,
   /* This is taken from Totem’s bacon-video-widget.c
      Changing the property immediately will crash, it has to be an idle source */
 
-  guint id;
-  id = g_idle_add ((GSourceFunc) notify_volume_idle_cb, self);
-  g_source_set_name_by_id (id, "[parlatype] notify_volume_idle_cb");
+  PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+  priv->vol_changed_id = g_idle_add_once (notify_volume_idle_cb, self);
+  g_source_set_name_by_id (priv->vol_changed_id, "[parlatype] notify_volume_idle_cb");
 }
 
-static gboolean
-notify_mute_idle_cb (PtPlayer *self)
+static void
+notify_mute_idle_cb (gpointer user_data)
 {
-  if (!PT_IS_PLAYER (self))
-    return G_SOURCE_REMOVE;
-
+  PtPlayer        *self = PT_PLAYER (user_data);
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   gboolean         mute;
 
@@ -2040,7 +2101,7 @@ notify_mute_idle_cb (PtPlayer *self)
                                 obj_properties[PROP_MUTE]);
     }
 
-  return FALSE;
+  priv->mute_changed_id = 0;
 }
 
 static void
@@ -2048,17 +2109,20 @@ mute_changed (GObject    *object,
               GParamSpec *pspec,
               PtPlayer   *self)
 {
-  guint id;
-  id = g_idle_add ((GSourceFunc) notify_mute_idle_cb, self);
-  g_source_set_name_by_id (id, "[parlatype] notify_mute_idle_cb");
+  PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+
+  priv->mute_changed_id = g_idle_add_once (notify_mute_idle_cb, self);
+  g_source_set_name_by_id (priv->mute_changed_id, "[parlatype] notify_mute_idle_cb");
 }
 
-static gboolean
-notify_uri_idle_cb (PtPlayer *self)
+static void
+notify_uri_idle_cb (gpointer user_data)
 {
+  PtPlayer        *self = PT_PLAYER (user_data);
+  PtPlayerPrivate *priv = pt_player_get_instance_private (self);
   g_object_notify_by_pspec (G_OBJECT (self),
                             obj_properties[PROP_CURRENT_URI]);
-  return FALSE;
+  priv->uri_changed_id = 0;
 }
 
 static void
@@ -2066,9 +2130,10 @@ uri_changed (GObject    *object,
              GParamSpec *pspec,
              PtPlayer   *self)
 {
-  guint id;
-  id = g_idle_add ((GSourceFunc) notify_uri_idle_cb, self);
-  g_source_set_name_by_id (id, "[parlatype] notify_uri_idle_cb");
+  PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+
+  priv->uri_changed_id = g_idle_add_once (notify_uri_idle_cb, self);
+  g_source_set_name_by_id (priv->uri_changed_id, "[parlatype] notify_uri_idle_cb");
 }
 
 /**
@@ -2204,6 +2269,26 @@ pt_player_get_mode (PtPlayer *self)
   return gst_pt_audio_bin_get_mode (bin);
 }
 
+/**
+ * pt_player_get_media_info:
+ * @self: a #PtPlayer
+ *
+ * Get the player’s #PtMediaInfo object to query the current
+ * track’s metadata.
+ *
+ * Return value: (transfer none): the player’s #PtMediaInfo object
+ *
+ * Since: 4.3
+ */
+PtMediaInfo *
+pt_player_get_media_info (PtPlayer *self)
+{
+  g_return_val_if_fail (PT_IS_PLAYER (self), NULL);
+  PtPlayerPrivate *priv = pt_player_get_instance_private (self);
+
+  return priv->info;
+}
+
 static void
 pt_player_constructed (GObject *object)
 {
@@ -2231,6 +2316,13 @@ pt_player_dispose (GObject *object)
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
 
   remove_seek_source (self);
+  g_clear_signal_handler (&priv->stream_notify_id, priv->collection);
+  g_clear_handle_id (&priv->vol_changed_id, g_source_remove);
+  g_clear_handle_id (&priv->mute_changed_id, g_source_remove);
+  g_clear_handle_id (&priv->uri_changed_id, g_source_remove);
+  g_clear_pointer (&priv->collection, gst_object_unref);
+  g_clear_object (&priv->info);
+
   if (priv->play)
     {
       /* remember position */
@@ -2255,6 +2347,7 @@ pt_player_finalize (GObject *object)
   PtPlayerPrivate *priv = pt_player_get_instance_private (self);
 
   g_mutex_clear (&priv->lock);
+  g_free (priv->stream_id);
 
   G_OBJECT_CLASS (pt_player_parent_class)->finalize (object);
 }
@@ -2427,7 +2520,7 @@ pt_player_init (PtPlayer *self)
     gst_pt_audio_bin_register ();
   else
     gst_object_unref (factory);
-#ifdef HAVE_POCKETSPHINX
+#if defined HAVE_POCKETSPHINX || defined HAVE_POCKETSPHINX_LEGACY
   factory = gst_element_factory_find ("parlasphinx");
   if (factory == NULL)
     gst_parlasphinx_register ();
@@ -2435,7 +2528,7 @@ pt_player_init (PtPlayer *self)
     gst_object_unref (factory);
 #endif
 
-  priv->play = _pt_make_element ("playbin", "play", NULL);
+  priv->play = _pt_make_element ("playbin3", "play", NULL);
   priv->scaletempo = _pt_make_element ("scaletempo", "tempo", NULL);
   priv->audio_bin = _pt_make_element ("ptaudiobin", "audiobin", NULL);
 
@@ -2445,6 +2538,7 @@ pt_player_init (PtPlayer *self)
 
   priv->current_state = GST_STATE_NULL;
   priv->target_state = GST_STATE_NULL;
+  priv->info = _pt_media_info_new ();
 }
 
 static void
@@ -2452,7 +2546,7 @@ pt_player_class_init (PtPlayerClass *klass)
 {
   G_OBJECT_CLASS (klass)->set_property = pt_player_set_property;
   G_OBJECT_CLASS (klass)->get_property = pt_player_get_property;
-  G_OBJECT_CLASS (klass)->dispose = pt_player_constructed;
+  G_OBJECT_CLASS (klass)->constructed = pt_player_constructed;
   G_OBJECT_CLASS (klass)->dispose = pt_player_dispose;
   G_OBJECT_CLASS (klass)->finalize = pt_player_finalize;
 
