@@ -21,6 +21,7 @@
 #endif
 
 #include "pt-config.h"
+#include "pt-media-info.h"
 #include "pt-waveviewer.h"
 
 G_BEGIN_DECLS
@@ -82,6 +83,7 @@ typedef enum
   PT_PRECISION_INVALID
 } PtPrecisionType;
 
+
 void       pt_player_pause                    (PtPlayer       *self);
 
 void       pt_player_pause_and_rewind         (PtPlayer       *self);
@@ -182,6 +184,8 @@ gboolean   pt_player_configure_asr            (PtPlayer       *self,
 gboolean   pt_player_config_is_loadable       (PtPlayer       *self,
                                                PtConfig       *config);
 
+PtMediaInfo *pt_player_get_media_info         (PtPlayer       *self);
+
 PtPlayer  *pt_player_new                      (void);
 
 G_END_DECLS
