@@ -21,7 +21,7 @@
  * SECTION: pt-waveloader
  * @short_description: Loads the waveform for a given file.
  * @stability: Stable
- * @include: parlatype/pt-waveloader.h
+ * @include: parlatype/parlatype.h
  *
  * An object to load waveform data from an audio file. The raw data can be
  * used by a widget to visually represent a waveform.
@@ -48,10 +48,10 @@ struct _PtWaveloaderPrivate
   GstElement *fmt;
 
   GArray *hires;
-  gint    hires_index;
+  uint    hires_index;
   GArray *lowres;
-  gint    pps;
-  gint    lowres_index;
+  int     pps;
+  uint    lowres_index;
 
   gchar   *uri;
   gboolean load_pending;
@@ -59,8 +59,8 @@ struct _PtWaveloaderPrivate
 
   gint64 duration;
 
-  gint    bus_watch_id;
-  gint    progress_timeout;
+  guint   bus_watch_id;
+  guint   progress_timeout;
   gdouble progress;
 };
 
@@ -82,18 +82,6 @@ static guint       signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (PtWaveloader, pt_waveloader, G_TYPE_OBJECT)
 
-static void
-remove_timeout (PtWaveloader *self)
-{
-  PtWaveloaderPrivate *priv = pt_waveloader_get_instance_private (self);
-
-  if (priv->progress_timeout > 0)
-    {
-      g_source_remove (priv->progress_timeout);
-      priv->progress_timeout = 0;
-    }
-}
-
 static void
 on_wave_loader_new_pad (GstElement *bin,
                         GstPad     *pad,
@@ -148,10 +136,13 @@ calc_lowres_len (gint hires_len,
 static void
 convert_one_second (GArray *in,
                     GArray *out,
-                    gint   *index_in,
-                    gint   *index_out,
-                    gint    pps)
+                    uint   *index_in,
+                    uint   *index_out,
+                    int     pps)
 {
+  g_return_if_fail (in != NULL);
+  g_return_if_fail (out != NULL);
+
   gint   k, m;
   gint16 d;
   gfloat vmin, vmax;
@@ -318,15 +309,14 @@ check_progress (GTask *task)
   gint64  dur;
   gint64  pos;
   gdouble temp;
-  gint    new_size;
+  uint    new_size;
 
   /* Check if task was cancelled and reset pipeline */
 
   if (g_cancellable_is_cancelled (g_task_get_cancellable (task)))
     {
       gst_element_set_state (priv->pipeline, GST_STATE_NULL);
-      g_source_remove (priv->bus_watch_id);
-      priv->bus_watch_id = 0;
+      g_clear_handle_id (&priv->bus_watch_id, g_source_remove);
       priv->progress_timeout = 0;
       g_array_set_size (priv->lowres, 0);
       g_task_return_boolean (task, FALSE);
@@ -383,7 +373,7 @@ bus_handler (GstBus     *bus,
         gchar  *debug;
         GError *error;
 
-        remove_timeout (self);
+        g_clear_handle_id (&priv->progress_timeout, g_source_remove);
         gst_message_parse_error (msg, &error, &debug);
 
         /* Error is returned. Log the message here at level DEBUG,
@@ -426,7 +416,7 @@ bus_handler (GstBus     *bus,
                           "MESSAGE", "Sample decoded: hires->len=%d, lowres->len=%d, pps=%d, duration=%" GST_TIME_FORMAT,
                           priv->hires->len, priv->lowres->len, priv->pps, GST_TIME_ARGS (priv->duration));
 
-        remove_timeout (self);
+        g_clear_handle_id (&priv->progress_timeout, g_source_remove);
         priv->bus_watch_id = 0;
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
@@ -482,8 +472,8 @@ pt_waveloader_load_finish (PtWaveloader *self,
  * @self: a #PtWaveloader
  * @pps: pixels per second
  * @cancellable: (nullable): a #GCancellable or NULL
- * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
- * @user_data: (closure): user_data for callback
+ * @callback: (scope async) (closure user_data): a #GAsyncReadyCallback to call when the operation is complete
+ * @user_data: user_data for @callback
  *
  * Saves sample data to private memory. To keep the memory footprint low, the
  * raw data is downsampled to 8000 samples per second (16 bit per sample). This
@@ -614,9 +604,9 @@ pt_waveloader_resize_real (GTask        *task,
   PtWaveloader        *self = source_object;
   PtWaveloaderPrivate *priv = pt_waveloader_get_instance_private (self);
   gint                 pps = GPOINTER_TO_INT (task_data);
-  gint                 lowres_len;
-  gint                 index_in = 0;
-  gint                 index_out = 0;
+  uint                 lowres_len;
+  uint                 index_in = 0;
+  uint                 index_out = 0;
   gboolean             result = TRUE;
 
   lowres_len = calc_lowres_len (priv->hires->len, pps);
@@ -665,8 +655,8 @@ pt_waveloader_resize_real (GTask        *task,
  * @self: a #PtWaveloader
  * @pps: the requested pixel per second ratio
  * @cancellable: (nullable): a #GCancellable or NULL
- * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
- * @user_data: (closure): user_data for callback
+ * @callback: (scope async) (closure user_data): a #GAsyncReadyCallback to call when the operation is complete
+ * @user_data: user_data for @callback
  *
  * Resizes wave form data at the requested resolution @pps. If the array's
  * size has to be changed, #PtWaveloader::array-size-changed is emitted.
@@ -840,8 +830,6 @@ pt_waveloader_init (PtWaveloader *self)
                              TRUE,  /* clear to zero   */
                              sizeof (gint16));
   priv->lowres = g_array_new (FALSE, TRUE, sizeof (float));
-  priv->bus_watch_id = 0;
-  priv->progress_timeout = 0;
   priv->load_pending = FALSE;
   priv->data_pending = FALSE;
 }
@@ -857,13 +845,8 @@ pt_waveloader_dispose (GObject *object)
   g_array_unref (priv->hires);
   g_array_unref (priv->lowres);
 
-  if (priv->bus_watch_id > 0)
-    {
-      g_source_remove (priv->bus_watch_id);
-      priv->bus_watch_id = 0;
-    }
-
-  remove_timeout (self);
+  g_clear_handle_id (&priv->bus_watch_id, g_source_remove);
+  g_clear_handle_id (&priv->progress_timeout, g_source_remove);
 
   if (priv->pipeline)
     {
