@@ -27,7 +27,7 @@
  * SECTION: pt-waveviewer
  * @short_description: A GtkWidget to display a waveform.
  * @stability: Stable
- * @include: parlatype/pt-waveviewer.h
+ * @include: parlatype/parlatype.h
  *
  * Displays a waveform and lets the user interact with it: jumping to a position,
  * make selections and so on.
@@ -81,8 +81,10 @@ struct _PtWaveviewerPrivate
   /* Selections, in milliseconds */
   gint64     sel_start;
   gint64     sel_end;
-  gint64     dragstart;
-  gint64     dragend;
+  gdouble    drag_start_0;
+  gdouble    drag_start_x;
+  gint64     drag_fixed_pos;
+  gint64     drag_moving_pos;
   GdkCursor *arrows;
 
   GtkAdjustment *adj;
@@ -104,9 +106,6 @@ struct _PtWaveviewerPrivate
   GtkEventController *key_ctrl;
   GtkEventController *focus_ctrl;
 
-  gdouble x_motion;
-  gdouble y_motion;
-
   guint tick_handler;
 };
 
@@ -253,14 +252,12 @@ static void
 update_selection (PtWaveviewer *self)
 {
   /* Check if drag positions are different from selection.
-     If yes, set new selection, emit signals and redraw widget. */
+     If yes, set new selection and emit signals. */
 
-  PtWaveviewerPrivate   *priv = pt_waveviewer_get_instance_private (self);
-  gboolean               changed = FALSE;
-  PtWaveviewerSelection *sel_widget = PT_WAVEVIEWER_SELECTION (priv->selection);
+  PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
 
   /* Is anything selected at all? */
-  if (priv->dragstart == priv->dragend)
+  if (priv->drag_fixed_pos == priv->drag_moving_pos)
     {
       priv->sel_start = 0;
       priv->sel_end = 0;
@@ -270,51 +267,39 @@ update_selection (PtWaveviewer *self)
           g_object_notify_by_pspec (G_OBJECT (self),
                                     obj_properties[PROP_HAS_SELECTION]);
           g_signal_emit_by_name (self, "selection-changed");
-          pt_waveviewer_selection_set (sel_widget, 0, 0);
         }
       return;
     }
 
-  /* Don’t select too much */
-  priv->dragstart = CLAMP (priv->dragstart, 0, priv->duration);
-  priv->dragend = CLAMP (priv->dragend, 0, priv->duration);
-
-  /* Sort start/end, check for changes (no changes on vertical movement), update selection */
-  if (priv->dragstart < priv->dragend)
+  /* Sort start/end, check for changes, update selection */
+  if (priv->drag_fixed_pos < priv->drag_moving_pos)
     {
-      if (priv->sel_start != priv->dragstart || priv->sel_end != priv->dragend)
+      if (priv->sel_start == priv->drag_fixed_pos && priv->sel_end == priv->drag_moving_pos)
         {
-          priv->sel_start = priv->dragstart;
-          priv->sel_end = priv->dragend;
-          changed = TRUE;
+          return;
         }
+      priv->sel_start = priv->drag_fixed_pos;
+      priv->sel_end = priv->drag_moving_pos;
     }
   else
     {
-      if (priv->sel_start != priv->dragend || priv->sel_end != priv->dragstart)
+      if (priv->sel_start == priv->drag_moving_pos && priv->sel_end == priv->drag_fixed_pos)
         {
-          priv->sel_start = priv->dragend;
-          priv->sel_end = priv->dragstart;
-          changed = TRUE;
+          return;
         }
+      priv->sel_start = priv->drag_moving_pos;
+      priv->sel_end = priv->drag_fixed_pos;
     }
 
-  if (changed)
+  /* Update has-selection property */
+  if (!priv->has_selection)
     {
-
-      /* Update has-selection property */
-      if (!priv->has_selection)
-        {
-          priv->has_selection = TRUE;
-          g_object_notify_by_pspec (G_OBJECT (self),
-                                    obj_properties[PROP_HAS_SELECTION]);
-        }
-
-      g_signal_emit_by_name (self, "selection-changed");
-      pt_waveviewer_selection_set (sel_widget,
-                                   time_to_pixel (self, priv->sel_start),
-                                   time_to_pixel (self, priv->sel_end));
+      priv->has_selection = TRUE;
+      g_object_notify_by_pspec (G_OBJECT (self),
+                                obj_properties[PROP_HAS_SELECTION]);
     }
+
+  g_signal_emit_by_name (self, "selection-changed");
 }
 
 static gboolean
@@ -336,7 +321,9 @@ pt_waveviewer_key_press_event (GtkEventControllerKey *ctrl,
       switch (keyval)
         {
         case GDK_KEY_Escape:
-          priv->dragstart = priv->dragend = 0;
+          priv->drag_fixed_pos = priv->drag_moving_pos = 0;
+          gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), NULL);
+          pt_waveviewer_selection_set (PT_WAVEVIEWER_SELECTION (priv->selection), 0, 0);
           update_selection (self);
           return TRUE;
         case GDK_KEY_space:
@@ -366,6 +353,8 @@ pt_waveviewer_key_press_event (GtkEventControllerKey *ctrl,
           else
             g_signal_emit_by_name (self, "cursor-changed", priv->duration);
           return TRUE;
+        default:
+          return FALSE;
         }
     }
 
@@ -393,14 +382,14 @@ pt_waveviewer_key_press_event (GtkEventControllerKey *ctrl,
         case GDK_KEY_End:
           scroll = GTK_SCROLL_END;
           break;
+        default:
+          return FALSE;
         }
-      if (scroll != GTK_SCROLL_NONE)
-        {
-          gboolean ret;
-          g_signal_emit_by_name (priv->scrolled_window, "scroll-child", scroll, TRUE, &ret);
-          pt_waveviewer_set_follow_cursor (self, FALSE);
-          return TRUE;
-        }
+
+      gboolean ret;
+      g_signal_emit_by_name (priv->scrolled_window, "scroll-child", scroll, TRUE, &ret);
+      pt_waveviewer_set_follow_cursor (self, FALSE);
+      return TRUE;
     }
 
   return FALSE;
@@ -427,54 +416,35 @@ pt_waveviewer_button_press_event (GtkGestureClick *gesture,
   PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
   GdkModifierType      state;
   guint                button;
-  gint64               clicked; /* the sample clicked on */
-  gint64               pos;     /* clicked sample’s position in milliseconds */
+  gint64               pos; /* clicked sample’s position in milliseconds */
 
   if (priv->peaks == NULL || priv->peaks->len == 0)
     return FALSE;
 
   state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (gesture));
-
   button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-  clicked = (gint) x;
-  pos = pixel_to_time (self, clicked + gtk_adjustment_get_value (priv->adj));
+  pos = pixel_to_time (self, (gint) x + gtk_adjustment_get_value (priv->adj));
 
-  /* Single left click, no other keys pressed: new selection or changing selection */
-  if (n_press == 1 && button == GDK_BUTTON_PRIMARY && !(state & ALL_ACCELS_MASK))
+  /* Single left click with Shift pressed and existing selection: change selection */
+  if (n_press == 1 &&
+      button == GDK_BUTTON_PRIMARY &&
+      (state & ALL_ACCELS_MASK) == GDK_SHIFT_MASK &&
+      priv->sel_start != priv->sel_end)
     {
-      /* set position as start and end point, for new selection */
-      priv->dragstart = priv->dragend = pos;
-
-      /* if over selection border: snap to selection border, changing selection */
-      if (pointer_in_range (self, x + gtk_adjustment_get_value (priv->adj), priv->sel_start))
-        {
-          priv->dragstart = priv->sel_end;
-          priv->dragend = priv->sel_start;
-        }
-      else if (pointer_in_range (self, x + gtk_adjustment_get_value (priv->adj), priv->sel_end))
-        {
-          priv->dragstart = priv->sel_start;
-          priv->dragend = priv->sel_end;
-        }
-
-      gtk_widget_set_cursor (GTK_WIDGET (self), priv->arrows);
-      update_selection (self);
-      return TRUE;
-    }
-
-  /* Single left click with Shift pressed and existing selection: enlarge selection */
-  if (n_press == 1 && button == GDK_BUTTON_PRIMARY && (state & ALL_ACCELS_MASK) == GDK_SHIFT_MASK && priv->sel_start != priv->sel_end)
-    {
-
-      priv->dragend = pos;
-
-      if (pos < priv->sel_start)
-        priv->dragstart = priv->sel_end;
+      /* Change selection: move the end that is closer to the clicked pixel.
+       * This can make the selection larger or smaller. */
+      pos = CLAMP (pos, 0, priv->duration);
+      if (ABS (priv->drag_fixed_pos - pos) < ABS (priv->drag_moving_pos - pos))
+        priv->drag_fixed_pos = pos;
       else
-        priv->dragstart = priv->sel_start;
+        priv->drag_moving_pos = pos;
 
-      gtk_widget_set_cursor (GTK_WIDGET (self), priv->arrows);
+      gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), priv->arrows);
+      pt_waveviewer_selection_set (PT_WAVEVIEWER_SELECTION (priv->selection),
+                                   time_to_pixel (self, priv->drag_fixed_pos),
+                                   time_to_pixel (self, priv->drag_moving_pos));
       update_selection (self);
+      gtk_widget_grab_focus (GTK_WIDGET (self));
       return TRUE;
     }
 
@@ -482,6 +452,7 @@ pt_waveviewer_button_press_event (GtkGestureClick *gesture,
   if (n_press == 1 && button == GDK_BUTTON_SECONDARY)
     {
       g_signal_emit_by_name (self, "cursor-changed", pos);
+      gtk_widget_grab_focus (GTK_WIDGET (self));
       return TRUE;
     }
 
@@ -534,7 +505,7 @@ pt_waveviewer_scroll_event (GtkEventControllerScroll *ctrl,
   return FALSE;
 }
 
-static gboolean
+static void
 pt_waveviewer_motion_event (GtkEventControllerMotion *ctrl,
                             gdouble                   x,
                             gdouble                   y,
@@ -542,66 +513,111 @@ pt_waveviewer_motion_event (GtkEventControllerMotion *ctrl,
 {
   PtWaveviewer        *self = PT_WAVEVIEWER (user_data);
   PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
-  GdkModifierType      state;
-  gint64               clicked; /* the sample clicked on */
-  gint64               pos;     /* clicked sample’s position in milliseconds */
-
-  /* TODO bug in GTK? investigate
-   * We get constantly motion events when playing, everything is normal in paused state.
-   * Filter out motion without changed x y */
-  if (x == priv->x_motion && y == priv->y_motion)
-    return FALSE;
-  priv->x_motion = x;
-  priv->y_motion = y;
-
-  state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (ctrl));
+  gboolean             have_selection, currently_dragging;
+  gdouble              real_x;
 
   if (priv->peaks == NULL || priv->peaks->len == 0)
-    return FALSE;
+    return;
 
-  clicked = (gint) x;
-  pos = pixel_to_time (self, clicked + gtk_adjustment_get_value (priv->adj));
+  have_selection = (priv->sel_start != priv->sel_end);
+  currently_dragging = (priv->drag_start_x >= 0);
+  real_x = x + gtk_adjustment_get_value (priv->adj);
 
-  /* Left mouse button (with or without Shift key) sets selection */
-  if (state & GDK_BUTTON1_MASK || state & GDK_BUTTON1_MASK & GDK_SHIFT_MASK)
+  if (!have_selection || currently_dragging)
+    return;
+
+  /* Change pointer cursor over selection border. */
+  if (pointer_in_range (self, real_x, priv->sel_start) ||
+      pointer_in_range (self, real_x, priv->sel_end))
     {
-      priv->dragend = pos;
-      update_selection (self);
-      return TRUE;
+      gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), priv->arrows);
     }
-
-  /* No button or any other button: change pointer cursor over selection border */
-  if (priv->sel_start != priv->sel_end)
+  else
     {
-      if (pointer_in_range (self, x + gtk_adjustment_get_value (priv->adj), priv->sel_start) || pointer_in_range (self, x + gtk_adjustment_get_value (priv->adj), priv->sel_end))
-        {
-          gtk_widget_set_cursor (GTK_WIDGET (self), priv->arrows);
-        }
-      else
-        {
-          gtk_widget_set_cursor (GTK_WIDGET (self), NULL);
-        }
+      gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), NULL);
     }
-
-  return FALSE;
 }
 
-static gboolean
-pt_waveviewer_button_release_event (GtkGestureClick *gesture,
-                                    gint             n_press,
-                                    gdouble          x,
-                                    gdouble          y,
-                                    gpointer         user_data)
+static void
+pt_waveviewer_drag_begin (GtkGestureDrag *ctrl,
+                          gdouble         start_x,
+                          gdouble         start_y,
+                          gpointer        user_data)
 {
-  guint button;
+  PtWaveviewer        *self = PT_WAVEVIEWER (user_data);
+  PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
+  gint64               pos; /* clicked sample’s position in milliseconds */
+  gdouble              x;
 
-  button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-  if (n_press == 1 && button == GDK_BUTTON_PRIMARY)
+  if (priv->peaks == NULL || priv->peaks->len == 0)
+    return;
+
+  /* Set position as start and end point for a new selection. */
+  priv->drag_start_0 = gtk_adjustment_get_value (priv->adj);
+  priv->drag_start_x = start_x;
+  x = priv->drag_start_0 + priv->drag_start_x;
+  pos = CLAMP (pixel_to_time (self, x), 0, priv->duration);
+  priv->drag_fixed_pos = priv->drag_moving_pos = pos;
+
+  /* if over selection border: snap to selection border, changing selection */
+  if (pointer_in_range (self, x, priv->sel_start))
     {
-      gtk_widget_set_cursor (GTK_WIDGET (user_data), NULL);
-      return TRUE;
+      priv->drag_moving_pos = priv->sel_start;
+      priv->drag_fixed_pos = priv->sel_end;
     }
-  return FALSE;
+  else if (pointer_in_range (self, x, priv->sel_end))
+    {
+      priv->drag_moving_pos = priv->sel_end;
+      priv->drag_fixed_pos = priv->sel_start;
+    }
+
+  gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), priv->arrows);
+  gtk_widget_grab_focus (GTK_WIDGET (self));
+  pt_waveviewer_selection_set (PT_WAVEVIEWER_SELECTION (priv->selection),
+                               time_to_pixel (self, priv->drag_fixed_pos),
+                               time_to_pixel (self, priv->drag_moving_pos));
+}
+
+static void
+pt_waveviewer_drag_update (GtkGestureDrag *ctrl,
+                           gdouble         offset_x,
+                           gdouble         offset_y,
+                           gpointer        user_data)
+{
+  PtWaveviewer        *self = PT_WAVEVIEWER (user_data);
+  PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
+  gint64               pixel;
+
+  if (priv->peaks == NULL || priv->peaks->len == 0)
+    return;
+
+  pixel = gtk_adjustment_get_value (priv->adj) + priv->drag_start_x + offset_x;
+  priv->drag_moving_pos = CLAMP (pixel_to_time (self, pixel), 0, priv->duration);
+
+  pt_waveviewer_selection_set (PT_WAVEVIEWER_SELECTION (priv->selection),
+                               time_to_pixel (self, priv->drag_fixed_pos),
+                               time_to_pixel (self, priv->drag_moving_pos));
+}
+
+static void
+pt_waveviewer_drag_end (GtkGestureDrag *ctrl,
+                        gdouble         offset_x,
+                        gdouble         offset_y,
+                        gpointer        user_data)
+{
+  PtWaveviewer        *self = PT_WAVEVIEWER (user_data);
+  PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
+
+  if (priv->peaks == NULL || priv->peaks->len == 0)
+    return;
+
+  /* Serves as in-drag-indicator. */
+  priv->drag_start_x = -1;
+
+  update_selection (self);
+
+  if (priv->sel_start == priv->sel_end)
+    gtk_widget_set_cursor (GTK_WIDGET (priv->cursor), NULL);
 }
 
 static gboolean
@@ -928,8 +944,8 @@ update_waveform_cb (GtkWidget     *widget,
  * @self: the widget
  * @uri: the URI of the file
  * @cancel: (nullable): a #GCancellable or NULL
- * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
- * @user_data: (closure): user data for callback
+ * @callback: (scope async) (closure user_data): a #GAsyncReadyCallback to call when the operation is complete
+ * @user_data: user data for @callback
  *
  * Load wave form for the given URI. The initial resolution is set to
  * #PtWaveviewer:pps. While loading, a #PtWaveviewer::load-progress signal is
@@ -1030,10 +1046,9 @@ pt_waveviewer_finalize (GObject *object)
 static void
 pt_waveviewer_dispose (GObject *object)
 {
-  PtWaveviewer        *self = PT_WAVEVIEWER (object);
-  PtWaveviewerPrivate *priv = pt_waveviewer_get_instance_private (self);
+  PtWaveviewer *self = PT_WAVEVIEWER (object);
 
-  g_clear_pointer (&priv->scrolled_window, gtk_widget_unparent);
+  gtk_widget_dispose_template (GTK_WIDGET (self), PT_TYPE_WAVEVIEWER);
 
   G_OBJECT_CLASS (pt_waveviewer_parent_class)->dispose (object);
 }
@@ -1210,6 +1225,7 @@ pt_waveviewer_init (PtWaveviewer *self)
   priv->follow_cursor = TRUE;
   priv->sel_start = 0;
   priv->sel_end = 0;
+  priv->drag_start_x = -1;
   priv->zoom_time = 0;
   priv->zoom_pos = 0;
   priv->arrows = get_resize_cursor ();
@@ -1239,43 +1255,50 @@ pt_waveviewer_init (PtWaveviewer *self)
   gtk_gesture_single_set_exclusive (GTK_GESTURE_SINGLE (priv->button), TRUE);
   gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (priv->button), 0);
   gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (priv->button), GTK_PHASE_CAPTURE);
-  g_signal_connect (
-      priv->button,
-      "pressed",
-      G_CALLBACK (pt_waveviewer_button_press_event),
-      self);
-  g_signal_connect (
-      priv->button,
-      "released",
-      G_CALLBACK (pt_waveviewer_button_release_event),
-      self);
+  g_signal_connect (priv->button,
+                    "pressed",
+                    G_CALLBACK (pt_waveviewer_button_press_event),
+                    self);
   gtk_widget_add_controller (priv->scrollbox, GTK_EVENT_CONTROLLER (priv->button));
 
   priv->motion_ctrl = gtk_event_controller_motion_new ();
-  g_signal_connect (
-      priv->motion_ctrl,
-      "motion",
-      G_CALLBACK (pt_waveviewer_motion_event),
-      self);
-  gtk_widget_add_controller (GTK_WIDGET (self), priv->motion_ctrl);
+  g_signal_connect (priv->motion_ctrl,
+                    "motion",
+                    G_CALLBACK (pt_waveviewer_motion_event),
+                    self);
+  gtk_widget_add_controller (GTK_WIDGET (priv->cursor), priv->motion_ctrl);
 
   priv->scroll_ctrl = gtk_event_controller_scroll_new (GTK_EVENT_CONTROLLER_SCROLL_HORIZONTAL | GTK_EVENT_CONTROLLER_SCROLL_VERTICAL);
   gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (priv->scroll_ctrl), GTK_PHASE_CAPTURE);
-  g_signal_connect (
-      priv->scroll_ctrl,
-      "scroll",
-      G_CALLBACK (pt_waveviewer_scroll_event),
-      self);
+  g_signal_connect (priv->scroll_ctrl,
+                    "scroll",
+                    G_CALLBACK (pt_waveviewer_scroll_event),
+                    self);
   gtk_widget_add_controller (GTK_WIDGET (self), priv->scroll_ctrl);
 
   priv->key_ctrl = gtk_event_controller_key_new ();
-  g_signal_connect (
-      priv->key_ctrl,
-      "key-pressed",
-      G_CALLBACK (pt_waveviewer_key_press_event),
-      self);
+  g_signal_connect (priv->key_ctrl,
+                    "key-pressed",
+                    G_CALLBACK (pt_waveviewer_key_press_event),
+                    self);
   gtk_widget_add_controller (GTK_WIDGET (self), priv->key_ctrl);
 
+  GtkGesture *drag = gtk_gesture_drag_new ();
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (drag), GTK_PHASE_CAPTURE);
+  g_signal_connect (drag,
+                    "drag-begin",
+                    G_CALLBACK (pt_waveviewer_drag_begin),
+                    self);
+  g_signal_connect (drag,
+                    "drag-update",
+                    G_CALLBACK (pt_waveviewer_drag_update),
+                    self);
+  g_signal_connect (drag,
+                    "drag-end",
+                    G_CALLBACK (pt_waveviewer_drag_end),
+                    self);
+  gtk_widget_add_controller (GTK_WIDGET (priv->overlay), GTK_EVENT_CONTROLLER (drag));
+
   g_signal_connect (priv->loader,
                     "progress",
                     G_CALLBACK (propagate_progress_cb),
