@@ -20,9 +20,9 @@
 #include <parlatype.h>
 
 #define PT_TYPE_ASR_DIALOG (pt_asr_dialog_get_type ())
-G_DECLARE_FINAL_TYPE (PtAsrDialog, pt_asr_dialog, PT, ASR_DIALOG, AdwPreferencesWindow)
+G_DECLARE_FINAL_TYPE (PtAsrDialog, pt_asr_dialog, PT, ASR_DIALOG, AdwPreferencesDialog)
 
 void         pt_asr_dialog_set_config (PtAsrDialog *self,
                                        PtConfig    *config);
 
-PtAsrDialog *pt_asr_dialog_new        (GtkWindow   *parent);
+PtAsrDialog *pt_asr_dialog_new        (void);
