@@ -41,9 +41,10 @@ struct _PtMpris
 
   GHashTable *player_property_changes;
   guint       property_emit_id;
+  GMutex      lock;
 };
 
-G_DEFINE_TYPE (PtMpris, pt_mpris, PT_CONTROLLER_TYPE)
+G_DEFINE_TYPE (PtMpris, pt_mpris, PT_TYPE_CONTROLLER)
 
 static void
 emit_property_changes (PtMpris *self, GHashTable *changes, const char *interface)
@@ -99,12 +100,15 @@ emit_property_changes (PtMpris *self, GHashTable *changes, const char *interface
 static gboolean
 emit_properties_idle (PtMpris *self)
 {
+  /* Protect hash table from concurrent access. */
+  g_mutex_lock (&self->lock);
   if (self->player_property_changes != NULL)
     {
       emit_property_changes (self, self->player_property_changes, MPRIS_PLAYER_INTERFACE);
       g_hash_table_destroy (self->player_property_changes);
       self->player_property_changes = NULL;
     }
+  g_mutex_unlock (&self->lock);
 
   self->property_emit_id = 0;
   return FALSE;
@@ -115,11 +119,14 @@ add_player_property_change (PtMpris    *self,
                             const char *property,
                             GVariant   *value)
 {
+  /* Protect hash table from concurrent access. */
+  g_mutex_lock (&self->lock);
   if (self->player_property_changes == NULL)
     {
       self->player_property_changes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_variant_unref);
     }
   g_hash_table_insert (self->player_property_changes, g_strdup (property), g_variant_ref_sink (value));
+  g_mutex_unlock (&self->lock);
 
   if (self->property_emit_id == 0)
     {
@@ -278,7 +285,10 @@ build_track_metadata (PtMpris         *self,
                       GVariantBuilder *builder,
                       PtPlayer        *player)
 {
-  gchar *uri;
+  gchar       *uri;
+  PtMediaInfo *info;
+  gchar       *str;
+  GStrv        strv;
 
   uri = pt_player_get_uri (player);
 
@@ -289,6 +299,23 @@ build_track_metadata (PtMpris         *self,
       return;
     }
 
+  info = pt_player_get_media_info (player);
+
+  str = pt_media_info_get_album (info);
+  if (str)
+    g_variant_builder_add (builder, "{sv}", "xesam:album",
+                           g_variant_new_string (str));
+
+  strv = pt_media_info_get_artist (info);
+  if (strv)
+    g_variant_builder_add (builder, "{sv}", "xesam:artist",
+                           g_variant_new_strv ((void *) strv, -1));
+
+  str = pt_media_info_get_title (info);
+  if (str)
+    g_variant_builder_add (builder, "{sv}", "xesam:title",
+                           g_variant_new_string (str));
+
   g_variant_builder_add (builder, "{sv}", "mpris:trackid",
                          g_variant_new ("o", "/org/mpris/MediaPlayer2/Track/1"));
   g_variant_builder_add (builder, "{sv}", "xesam:url",
@@ -631,7 +658,7 @@ volume_changed_cb (GObject *object, GParamSpec *pspec, PtMpris *self)
 }
 
 static void
-uri_changed_cb (GObject *object, GParamSpec *pspec, PtMpris *self)
+metadata_property_change (PtMpris *self)
 {
   PtPlayer        *player = pt_controller_get_player (PT_CONTROLLER (self));
   GVariantBuilder *builder;
@@ -643,6 +670,18 @@ uri_changed_cb (GObject *object, GParamSpec *pspec, PtMpris *self)
   g_variant_builder_unref (builder);
 }
 
+static void
+uri_changed_cb (GObject *object, GParamSpec *pspec, PtMpris *self)
+{
+  metadata_property_change (self);
+}
+
+static void
+media_info_changed_cb (GObject *object, PtMpris *self)
+{
+  metadata_property_change (self);
+}
+
 static void
 name_acquired_cb (GDBusConnection *connection, const char *name, PtMpris *self)
 {
@@ -661,6 +700,7 @@ void
 pt_mpris_start (PtMpris *self)
 {
   PtPlayer           *player = pt_controller_get_player (PT_CONTROLLER (self));
+  PtMediaInfo        *info = pt_player_get_media_info (player);
   GDBusInterfaceInfo *ifaceinfo;
   GError             *error = NULL;
 
@@ -733,8 +773,13 @@ pt_mpris_start (PtMpris *self)
                            G_CALLBACK (uri_changed_cb),
                            self, 0);
 
+  g_signal_connect_object (info,
+                           "media-info-changed",
+                           G_CALLBACK (media_info_changed_cb),
+                           self, 0);
+
   self->name_own_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-                                      MPRIS_BUS_NAME_PREFIX ".parlatype",
+                                      MPRIS_BUS_NAME_PREFIX "." APP_ID,
                                       G_BUS_NAME_OWNER_FLAGS_NONE,
                                       NULL,
                                       (GBusNameAcquiredCallback) name_acquired_cb,
@@ -743,17 +788,15 @@ pt_mpris_start (PtMpris *self)
                                       g_object_unref);
 }
 
-static void
-pt_mpris_init (PtMpris *self)
-{
-}
-
 static void
 pt_mpris_dispose (GObject *object)
 {
-  PtMpris  *self = PT_MPRIS (object);
-  PtPlayer *player = pt_controller_get_player (PT_CONTROLLER (self));
+  PtMpris     *self = PT_MPRIS (object);
+  PtPlayer    *player = pt_controller_get_player (PT_CONTROLLER (self));
+  PtMediaInfo *info = pt_player_get_media_info (player);
 
+  g_mutex_clear (&self->lock);
+  g_clear_handle_id (&self->property_emit_id, g_source_remove);
   if (self->root_id != 0)
     {
       g_dbus_connection_unregister_object (self->connection, self->root_id);
@@ -781,9 +824,23 @@ pt_mpris_dispose (GObject *object)
                                         G_CALLBACK (uri_changed_cb),
                                         self);
 
+  g_signal_handlers_disconnect_by_func (player,
+                                        G_CALLBACK (uri_changed_cb),
+                                        self);
+
+  g_signal_handlers_disconnect_by_func (info,
+                                        G_CALLBACK (media_info_changed_cb),
+                                        self);
+
   G_OBJECT_CLASS (pt_mpris_parent_class)->dispose (object);
 }
 
+static void
+pt_mpris_init (PtMpris *self)
+{
+  g_mutex_init (&self->lock);
+}
+
 static void
 pt_mpris_class_init (PtMprisClass *klass)
 {
@@ -793,5 +850,5 @@ pt_mpris_class_init (PtMprisClass *klass)
 PtMpris *
 pt_mpris_new (PtWindow *win)
 {
-  return g_object_new (PT_MPRIS_TYPE, "win", win, NULL);
+  return g_object_new (PT_TYPE_MPRIS, "win", win, NULL);
 }
