@@ -31,15 +31,14 @@
 #define EXAMPLE_TIME_SHORT 471123
 #define EXAMPLE_TIME_LONG 4071123
 
-static GtkWidget *preferences_dialog = NULL;
-
 struct _PtPreferencesDialog
 {
-  AdwPreferencesWindow parent;
+  AdwPreferencesDialog parent;
 
   GSettings    *editor;
   PtPlayer     *player;
   PtConfigList *list;
+  GtkWindow    *parent_window;
 
   /* Waveform page */
   GtkWidget *pps_scale;
@@ -66,9 +65,11 @@ struct _PtPreferencesDialog
   GPtrArray *config_rows;
   gchar     *active_config_path;
   GtkWidget *initial_copy_button;
+
+  ulong handler_id;
 };
 
-G_DEFINE_FINAL_TYPE (PtPreferencesDialog, pt_preferences_dialog, ADW_TYPE_PREFERENCES_WINDOW)
+G_DEFINE_FINAL_TYPE (PtPreferencesDialog, pt_preferences_dialog, ADW_TYPE_PREFERENCES_DIALOG)
 
 static void
 update_timestamp_page (PtPreferencesDialog *self)
@@ -98,6 +99,7 @@ settings_changed_cb (GSettings *settings,
                      gpointer   user_data)
 {
   PtPreferencesDialog *self = PT_PREFERENCES_DIALOG (user_data);
+
   if (g_str_has_prefix (key, "timestamp"))
     {
       update_timestamp_page (self);
@@ -158,7 +160,7 @@ get_delimiter_mapping (GValue   *value,
                        gpointer  data)
 {
   const gchar *sep = g_variant_get_string (variant, NULL);
-  gint         drop_down_position;
+  int          drop_down_position;
   if (g_strcmp0 (sep, "None") == 0)
     drop_down_position = 0;
   else if (g_strcmp0 (sep, "#") == 0)
@@ -214,10 +216,10 @@ asr_row_activated (AdwActionRow *row,
   PtConfig            *config;
 
   config = pt_config_row_get_config (PT_CONFIG_ROW (row));
-  PtAsrDialog *asr_dlg = pt_asr_dialog_new (GTK_WINDOW (self));
+  PtAsrDialog *asr_dlg = pt_asr_dialog_new ();
   pt_asr_dialog_set_config (asr_dlg, config);
   g_signal_connect (asr_dlg, "destroy", G_CALLBACK (asr_dialog_destroy_cb), self);
-  gtk_window_present (GTK_WINDOW (asr_dlg));
+  adw_dialog_present (ADW_DIALOG (asr_dlg), GTK_WIDGET (self));
 }
 
 static gboolean
@@ -247,7 +249,7 @@ copy_asr_configs_result (GObject      *source_object,
       GtkAlertDialog *err_dialog;
       err_dialog = gtk_alert_dialog_new (_ ("Error"));
       gtk_alert_dialog_set_detail (err_dialog, error->message);
-      gtk_alert_dialog_show (err_dialog, GTK_WINDOW (self));
+      gtk_alert_dialog_show (err_dialog, self->parent_window);
       g_object_unref (err_dialog);
       g_error_free (error);
     }
@@ -404,7 +406,6 @@ import_copy_ready_cb (GObject      *source_object,
   GFile               *file = G_FILE (source_object);
   GError              *error = NULL;
   gboolean             success;
-  GtkWindow           *parent = GTK_WINDOW (self);
   GtkAlertDialog      *err_dialog;
 
   success = g_file_copy_finish (file, res, &error);
@@ -417,7 +418,7 @@ import_copy_ready_cb (GObject      *source_object,
     {
       err_dialog = gtk_alert_dialog_new (_ ("Error"));
       gtk_alert_dialog_set_detail (err_dialog, error->message);
-      gtk_alert_dialog_show (err_dialog, parent);
+      gtk_alert_dialog_show (err_dialog, self->parent_window);
       g_object_unref (err_dialog);
       g_error_free (error);
     }
@@ -431,7 +432,6 @@ dialog_open_cb (GObject      *source,
   PtPreferencesDialog *self = PT_PREFERENCES_DIALOG (user_data);
   GFile               *file;
   PtConfig            *config;
-  GtkWindow           *parent = GTK_WINDOW (self);
   GtkAlertDialog      *err_dialog;
   GFile               *dest_folder;
   gchar               *dest_folder_path;
@@ -463,7 +463,7 @@ dialog_open_cb (GObject      *source,
 
       gtk_alert_dialog_set_detail (err_dialog, message);
 
-      gtk_alert_dialog_show (err_dialog, parent);
+      gtk_alert_dialog_show (err_dialog, self->parent_window);
       g_object_unref (err_dialog);
       g_object_unref (config);
       return;
@@ -515,7 +515,6 @@ import_button_clicked_cb (GtkButton *button,
 {
   PtPreferencesDialog *self = PT_PREFERENCES_DIALOG (user_data);
   GtkFileDialog       *dialog;
-  GtkWindow           *parent = GTK_WINDOW (self);
   const char          *home_path;
   GFile               *initial_folder;
   GListStore          *filters;
@@ -541,7 +540,7 @@ import_button_clicked_cb (GtkButton *button,
   g_list_store_append (filters, filter_all);
   gtk_file_dialog_set_filters (dialog, G_LIST_MODEL (filters));
 
-  gtk_file_dialog_open (dialog, parent,
+  gtk_file_dialog_open (dialog, self->parent_window,
                         NULL, /* cancellable */
                         dialog_open_cb,
                         self);
@@ -555,6 +554,7 @@ pt_preferences_dialog_init (PtPreferencesDialog *self)
 {
   self->editor = g_settings_new (APP_ID);
   self->player = pt_player_new ();
+  self->parent_window = GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self)));
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -654,7 +654,7 @@ pt_preferences_dialog_init (PtPreferencesDialog *self)
       set_delimiter_mapping,
       NULL, NULL);
 
-  g_signal_connect (
+  self->handler_id = g_signal_connect (
       self->editor, "changed",
       G_CALLBACK (settings_changed_cb),
       self);
@@ -670,7 +670,7 @@ pt_preferences_dialog_init (PtPreferencesDialog *self)
   self->active_config_path = g_settings_get_string (self->editor, "asr-config");
 
 #ifndef HAVE_ASR
-  adw_preferences_window_remove (ADW_PREFERENCES_WINDOW (self),
+  adw_preferences_dialog_remove (ADW_PREFERENCES_DIALOG (self),
                                  ADW_PREFERENCES_PAGE (self->asr_page));
 #endif
 }
@@ -680,11 +680,18 @@ pt_preferences_dialog_dispose (GObject *object)
 {
   PtPreferencesDialog *self = PT_PREFERENCES_DIALOG (object);
 
+  /* Disconnect early because of race condition in screenshot tool */
+  g_clear_signal_handler (&self->handler_id, self->editor);
+
+  gtk_widget_unparent (GTK_WIDGET (self));
   g_clear_object (&self->editor);
   g_clear_object (&self->player);
-  g_ptr_array_free (self->config_rows, TRUE);
+  if (self->config_rows)
+    g_ptr_array_free (self->config_rows, TRUE);
+  g_clear_object (&self->list);
 
   g_free (self->active_config_path);
+  gtk_widget_dispose_template (GTK_WIDGET (self), PT_TYPE_PREFERENCES_DIALOG);
 
   G_OBJECT_CLASS (pt_preferences_dialog_parent_class)->dispose (object);
 }
@@ -736,42 +743,8 @@ pt_preferences_dialog_class_init (PtPreferencesDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PtPreferencesDialog, initial_copy_button);
 }
 
-static void
-prefs_destroy_cb (GtkWidget *widget,
-                  gpointer   user_data)
-{
-  preferences_dialog = NULL;
-}
-
-void
-pt_show_preferences_dialog (GtkWindow *parent)
-{
-  if (preferences_dialog == NULL)
-    {
-
-      preferences_dialog = GTK_WIDGET (g_object_new (PT_TYPE_PREFERENCES_DIALOG,
-                                                     NULL));
-      g_signal_connect (preferences_dialog,
-                        "destroy",
-                        G_CALLBACK (prefs_destroy_cb),
-                        NULL);
-    }
-
-  if (parent != gtk_window_get_transient_for (GTK_WINDOW (preferences_dialog)))
-    {
-      gtk_window_set_transient_for (GTK_WINDOW (preferences_dialog), GTK_WINDOW (parent));
-      gtk_window_set_modal (GTK_WINDOW (preferences_dialog), FALSE);
-      gtk_window_set_destroy_with_parent (GTK_WINDOW (preferences_dialog), TRUE);
-    }
-
-  gtk_window_present (GTK_WINDOW (preferences_dialog));
-}
-
 PtPreferencesDialog *
-pt_preferences_dialog_new (GtkWindow *parent)
+pt_preferences_dialog_new (void)
 {
-  return g_object_new (
-      PT_TYPE_PREFERENCES_DIALOG,
-      "transient-for", parent,
-      NULL);
+  return g_object_new (PT_TYPE_PREFERENCES_DIALOG, NULL);
 }
