@@ -1,4 +1,4 @@
-/* Copyright (C) Gabor Karsay 2016 <gabor.karsay@gmx.at>
+/* Copyright 2016 Gabor Karsay <gabor.karsay@gmx.at>
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as published
@@ -15,18 +15,21 @@
  */
 
 #include "config.h"
-#include <gtk/gtk.h>
-#include "pt-window.h"
-#include "pt-window-private.h"
+
 #include "pt-window-dnd.h"
 
+#include "pt-window-private.h"
+#include "pt-window.h"
+
+#include <gtk/gtk.h>
+
 static gboolean
 handle_uri (PtWindow *win,
-            gchar *data)
+            gchar    *data)
 {
-  GFile *file;
-  gchar **split = NULL;
-  gchar *uri;
+  GFile   *file;
+  gchar  **split = NULL;
+  gchar   *uri;
   gboolean success = FALSE;
 
   if (g_regex_match_simple ("^file:///.*", data, 0, 0))
@@ -71,16 +74,18 @@ handle_uri (PtWindow *win,
 
 static gboolean
 handle_filename (PtWindow *win,
-                 gchar *data)
+                 gchar    *data)
 {
-  gboolean success;
-  GList *list;
-  GtkRecentInfo *info;
-  gchar *uri;
+  gboolean          success;
+  GList            *list;
+  GtkRecentManager *recent;
+  GtkRecentInfo    *info;
+  gchar            *uri;
 
   success = FALSE;
 
-  list = gtk_recent_manager_get_items (win->priv->recent);
+  recent = _pt_window_get_recent_manager (win);
+  list = gtk_recent_manager_get_items (recent);
   for (list = g_list_first (list); list; list = g_list_next (list))
     {
       info = (GtkRecentInfo *) list->data;
@@ -110,12 +115,14 @@ handle_filename (PtWindow *win,
 
 static gboolean
 handle_dnd_data (PtWindow *win,
-                 gchar *data)
+                 gchar    *data)
 {
   g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
                     "MESSAGE", "Received drag and drop: '%s'", data);
 
-  if (pt_player_goto_timestamp (win->player, data))
+  PtPlayer *player = _pt_window_get_player (win);
+
+  if (pt_player_goto_timestamp (player, data))
     return TRUE;
 
   if (handle_uri (win, data))
@@ -129,10 +136,10 @@ handle_dnd_data (PtWindow *win,
 
 static gboolean
 drop_cb (GtkDropTarget *self,
-         GValue *value,
-         gdouble x,
-         gdouble y,
-         gpointer user_data)
+         GValue        *value,
+         gdouble        x,
+         gdouble        y,
+         gpointer       user_data)
 {
   PtWindow *win = PT_WINDOW (user_data);
 
