@@ -31,7 +31,7 @@
 
 struct _PtWindow
 {
-  GtkApplicationWindow parent;
+  AdwApplicationWindow parent;
   PtPlayer            *player;
   GtkWidget           *waveviewer;
   GSettings           *editor;
@@ -69,11 +69,11 @@ struct _PtWindow
 
   gint64 last_time; // last time to compare if it changed
 
-  gint    timer;
+  guint   timer;
   gdouble speed;
 };
 
-G_DEFINE_TYPE (PtWindow, pt_window, GTK_TYPE_APPLICATION_WINDOW)
+G_DEFINE_TYPE (PtWindow, pt_window, ADW_TYPE_APPLICATION_WINDOW)
 
 static void play_button_toggled_cb (GtkToggleButton *button, PtWindow *self);
 
@@ -141,11 +141,11 @@ insert_timestamp (GSimpleAction *action,
 
 static void
 goto_dialog_response_cb (GtkDialog *dlg,
-                         gint       response_id,
+                         int        response_id,
                          gpointer   user_data)
 {
   PtWindow *self = PT_WINDOW (user_data);
-  gint      pos;
+  int       pos;
 
   if (response_id == GTK_RESPONSE_OK)
     {
@@ -218,9 +218,9 @@ play (GSimpleAction *action,
 
 static void
 set_zoom (GSettings *editor,
-          gint       step)
+          int        step)
 {
-  gint pps;
+  int pps;
 
   pps = g_settings_get_int (editor, "pps");
   if ((step > 0 && pps >= 200) || (step < 0 && pps <= 25))
@@ -389,7 +389,7 @@ static void
 change_jump_back_tooltip (PtWindow *self)
 {
   gchar *back;
-  gint   seconds;
+  int    seconds;
 
   seconds = pt_player_get_back (self->player) / 1000;
   back = g_strdup_printf (
@@ -406,7 +406,7 @@ static void
 change_jump_forward_tooltip (PtWindow *self)
 {
   gchar *forward;
-  gint   seconds;
+  int    seconds;
 
   seconds = pt_player_get_forward (self->player) / 1000;
   forward = g_strdup_printf (
@@ -423,7 +423,7 @@ static void
 change_play_button_tooltip (PtWindow *self)
 {
   gchar   *play;
-  gint     pause;
+  int      pause;
   gboolean free_me = FALSE;
 
   pause = pt_player_get_pause (self->player) / 1000;
@@ -464,7 +464,7 @@ update_insert_action_sensitivity_cb (GdkClipboard *clip,
   gchar    *timestamp = NULL;
   gboolean  result = FALSE;
   GAction  *action;
-  gint      pos;
+  int       pos;
   gchar    *label;
 
   text = gdk_clipboard_read_text_finish (clip, res, NULL);
@@ -656,7 +656,7 @@ pt_window_open_file (PtWindow *self,
   g_return_if_fail (uri != NULL);
 
   gchar *current_uri;
-  gint   cmp = 1;
+  int    cmp = 1;
 
   /* Don't reload already loaded waveform */
   current_uri = pt_player_get_uri (self->player);
@@ -852,7 +852,7 @@ map_seconds_to_milliseconds (GValue   *value,
                              gpointer  data)
 {
   /* Settings store seconds, PtPlayer wants milliseconds */
-  gint new;
+  int new;
   new = g_variant_get_int32 (variant);
   new = new * 1000;
   g_value_set_int (value, new);
@@ -864,7 +864,7 @@ map_milliseconds_to_seconds (const GValue       *value,
                              const GVariantType *type,
                              gpointer            data)
 {
-  gint new;
+  int new;
   new = g_value_get_int (value);
   new = new / 1000;
   return g_variant_new_int32 (new);
@@ -1030,7 +1030,7 @@ volume_button_event_cb (GtkEventControllerLegacy *ctrl,
 
 static gboolean
 volume_button_press_event (GtkGestureClick *gesture,
-                           gint             n_press,
+                           int              n_press,
                            gdouble          x,
                            gdouble          y,
                            gpointer         user_data)
@@ -1274,7 +1274,6 @@ pt_window_init (PtWindow *self)
   self->recent = gtk_recent_manager_get_default ();
   self->timer = 0;
   self->last_time = 0;
-  self->clip_handler_id = 0;
   self->clip = gtk_widget_get_clipboard (GTK_WIDGET (self));
 
   /* Used e.g. by Xfce */
@@ -1305,8 +1304,7 @@ static void
 pt_window_dispose (GObject *object)
 {
   PtWindow *self = PT_WINDOW (object);
-  gint      x;
-  gint      y;
+  int       x, y;
 
   /* Save window size */
   if (self->editor)
@@ -1316,11 +1314,7 @@ pt_window_dispose (GObject *object)
       g_settings_set_int (self->editor, "height", y);
     }
 
-  if (self->clip_handler_id > 0)
-    {
-      g_signal_handler_disconnect (self->clip, self->clip_handler_id);
-      self->clip_handler_id = 0;
-    }
+  g_clear_signal_handler (&self->clip_handler_id, self->clip);
   remove_timer (self);
   g_clear_object (&self->editor);
   g_clear_object (&self->player);
@@ -1335,6 +1329,7 @@ pt_window_dispose (GObject *object)
       g_strfreev (self->vol_icons);
       self->vol_icons = NULL;
     }
+  gtk_widget_dispose_template (GTK_WIDGET (self), PT_TYPE_WINDOW);
 
   G_OBJECT_CLASS (pt_window_parent_class)->dispose (object);
 }
@@ -1370,5 +1365,5 @@ pt_window_class_init (PtWindowClass *klass)
 PtWindow *
 pt_window_new (PtApp *app)
 {
-  return g_object_new (PT_WINDOW_TYPE, "application", app, NULL);
+  return g_object_new (PT_TYPE_WINDOW, "application", app, NULL);
 }
