@@ -19,8 +19,8 @@
 #include "pt-app.h"
 #include <gtk/gtk.h>
 
-#define PT_WINDOW_TYPE (pt_window_get_type ())
-G_DECLARE_FINAL_TYPE (PtWindow, pt_window, PT, WINDOW, GtkApplicationWindow)
+#define PT_TYPE_WINDOW (pt_window_get_type ())
+G_DECLARE_FINAL_TYPE (PtWindow, pt_window, PT, WINDOW, AdwApplicationWindow)
 
 struct _PtWindowClass
 {
